@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 """iofile.py"""
 
-# Copyright (C) 2008, 2009, 2010, 2011, 2012, 2013, 2014 Federico Brega, Pierluigi Villani
+# Copyright (C) 2008-2017 Federico Brega, Pierluigi Villani
 
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -15,7 +15,6 @@
 
 from __future__ import with_statement
 import csv
-import codecs
 import logging
 import os
 import xml.dom.minidom
@@ -27,6 +26,7 @@ CGX_VERSION = "1.0"
 
 STAT_VER = 2
 
+
 def save_file(filename, slope):
     """Save .cgx file"""
     doc = xml.dom.minidom.Document()
@@ -66,16 +66,17 @@ def save_file(filename, slope):
         coords.appendChild(coord)
     sl.appendChild(coords)
 
-    with open(filename,"wb") as fid:
+    with open(filename, "w") as fid:
         doc.writexml(fid, encoding='utf-8', newl='\n')
 
+
 def __createTextElement(doc, name, text):
     elem = doc.createElement(name)
     ptext = doc.createTextNode(text)
     elem.appendChild(ptext)
     return elem
 
-#def save_file(filename, slope):
+# def save_file(filename, slope):
 #    """Save .csv file"""
 #    #Control if the right extension is given by user
 #    if not filename.endswith('.csv'):
@@ -98,14 +99,15 @@ def __createTextElement(doc, name, text):
 #        #    fid.write ("%.3f;%.3f;%s\n" % cpt)
 #    #fid.close()
 
+
 def open_file(filename, slopelist, slopenum, extension=''):
     """Open generic file"""
     statusok = True
     if not extension:
-        if isinstance(filename, basestring):
+        if isinstance(filename, str):
             extension = filename.split('.')[-1].lower()
         else:
-            extension = 'cgx' #when used stdin
+            extension = 'cgx'  # when used stdin
     if extension == 'cgx':
         try:
             statusok = __open_cgx(filename, slopelist, slopenum)
@@ -168,39 +170,25 @@ def open_file(filename, slopelist, slopenum, extension=''):
         statusok = False
     return statusok
 
-### Functions defining how to open a certain file format.
+# Functions defining how to open a certain file format.
 
-#csv unicode implementation, std library doesn't implements it yet :-(
-#from http://docs.python.org/library/csv.html#csv-examples
-def unicode_csv_reader(unicode_csv_data, dialect=csv.excel, **kwargs):
-    # csv.py doesn't do Unicode; encode temporarily as UTF-8:
-    csv_reader = csv.reader(utf_8_encoder(unicode_csv_data),
-                            dialect=dialect, **kwargs)
-    for row in csv_reader:
-        # decode UTF-8 back to Unicode, cell by cell:
-        yield [unicode(cell, 'utf-8') for cell in row]
-
-def utf_8_encoder(unicode_csv_data):
-    for line in unicode_csv_data:
-        yield line.encode('utf-8')
 
 def __open_csv(filename, slopelist, slopenum):
     """Open a Cyclograph CSV file (legacy)"""
-    if isinstance(filename, basestring):
-        fid = codecs.open(filename, encoding='utf-8', mode='rb')
+    if isinstance(filename, str):
+        fid = open(filename, encoding='utf-8', mode='r')
     else:
         fid = filename
-    #with open(filename, 'rb') as fid:
-    csv_reader = unicode_csv_reader(fid, delimiter=';', quotechar='\"')
-    line = csv_reader.next()
-    #Make it more robust: if file has less than the required elements
-    #it considers them as empty strings and doesn't consider exceding ones.
+    csv_reader = csv.reader(fid, delimiter=';', quotechar='\"')
+    line = next(csv_reader)
+    # Make it more robust: if file has less than the required elements
+    # it considers them as empty strings and doesn't consider exceding ones.
     if len(line) < 6:
-        line.extend([""] * (6-len(line)) )
+        line.extend([""] * (6-len(line)))
     (version, slopename, slopestate, slopeauthor, authoremail, slopecomment) = line[:6]
     status = True
-    
-    #This is a legacy format so always warn about that, regardless the version number
+
+    # This is a legacy format so always warn about that, regardless the version number
     status = STAT_VER
 
     slopelist.set_name(slopenum, slopename)
@@ -210,7 +198,7 @@ def __open_csv(filename, slopelist, slopenum):
     slopelist.set_comment(slopenum, slopecomment)
 
     for line in csv_reader:
-        #(dist, alt, name)=line.strip().split(';')
+        # (dist, alt, name)=line.strip().split(';')
         try:
             (dist, alt, name) = line
         except ValueError:
@@ -219,6 +207,7 @@ def __open_csv(filename, slopelist, slopenum):
     fid.close()
     return status
 
+
 def __open_cgx(filename, slopelist, slopenum):
     """Open a Cyclograph cgx file"""
     fid = xml.dom.minidom.parse(filename)
@@ -244,10 +233,10 @@ def __open_cgx(filename, slopelist, slopenum):
 
     for cpt in slop.getElementsByTagName('cp'):
         try:
-            slopelist.add_cp(slopenum, float(cpt.attributes["dist"].value)
-                                     , float(cpt.attributes["alt"].value)
-                                     , cpt.getAttribute("desc"))
-        except Exception, err:
+            slopelist.add_cp(slopenum, float(cpt.attributes["dist"].value),
+                                       float(cpt.attributes["alt"].value),
+                                       cpt.getAttribute("desc"))
+        except Exception as err:
             logging.exception(err)
             fid.unlink()
             return False
@@ -256,13 +245,14 @@ def __open_cgx(filename, slopelist, slopenum):
             slopelist.add_coord(slopenum,
                                 float(coord.attributes["lat"].value),
                                 float(coord.attributes["lon"].value))
-        except Exception, err:
+        except Exception as err:
             logging.exception(err)
             fid.unlink()
             return False
     fid.unlink()
     return True
 
+
 def __open_xml(filename, slopelist, slopenum):
     """Open a Cyclomaniac xml file"""
     fid = xml.dom.minidom.parse(filename)
@@ -273,9 +263,9 @@ def __open_xml(filename, slopelist, slopenum):
     for sample in samples.getElementsByTagName('sample'):
         for ent in sample.childNodes:
             if ent.localName == "distance":
-                dist = ent.childNodes[0].data.replace(',','.')
+                dist = ent.childNodes[0].data.replace(',', '.')
             elif ent.localName == "height":
-                alt = ent.childNodes[0].data.replace(',','.')
+                alt = ent.childNodes[0].data.replace(',', '.')
             elif ent.localName == "label":
                 name = ent.childNodes[0].data
         slopelist.add_cp(slopenum, float(dist), float(alt), name)
@@ -283,10 +273,11 @@ def __open_xml(filename, slopelist, slopenum):
     fid.unlink()
     return True
 
+
 def load_gpxfile(xmlfile):
     """Load a gpx file"""
     try:
-        if isinstance(xmlfile, basestring) and xmlfile.strip().startswith('<'):
+        if isinstance(xmlfile, str) and xmlfile.strip().startswith('<'):
             fid = xml.dom.minidom.parseString(xmlfile)
         else:
             fid = xml.dom.minidom.parse(xmlfile)
@@ -301,29 +292,29 @@ def load_gpxfile(xmlfile):
             if i == 0 or dist > next_cp or i == len(cps)-1:
                 indices.append(i)
                 next_cp = dist + max_dist/len(cps)
-        #Use waypoints to determine the name of a cp.
+        # Use waypoints to determine the name of a cp.
         waypoints = {}
         for waypoint in fid.getElementsByTagName('wpt'):
-            #find the nearest (filtered) point to this waypont
+            # find the nearest (filtered) point to this waypont
             (wpt_lon, wpt_lat) = point_conversion(waypoint.attributes["lon"].value,
                                                   waypoint.attributes["lat"].value)
-            (min_i, min_distance) = (-1, float(3)) #use minimum accetable distance 3 km.
-            #print cps
+            (min_i, min_distance) = (-1, float(3))  # use minimum accetable distance 3 km.
+            # print cps
             for i in indices:
-                #calculate distance to point i
+                # calculate distance to point i
                 (lon, lat) = point_conversion(cps[i][1], cps[i][2])
                 dist = haversine(lat, wpt_lat, lon, wpt_lon)
-                #print (lon, lat, dist)
+                # print (lon, lat, dist)
                 if dist < min_distance:
                     (min_i, min_distance) = (i, dist)
             if min_i >= 0:
-                #is found a point near enought to the waypoint: use it.
+                # is found a point near enought to the waypoint: use it.
                 waypoints[min_i] = waypoint.getElementsByTagName("name")[0].firstChild.data
         returncp = []
         is_with_alt = True
         alt_count = 0
         for i in indices:
-            #get elevation
+            # get elevation
             (dist, lng, lat) = cps[i]
             sample = ptls[i]
             elelist = sample.getElementsByTagName('ele')
@@ -347,14 +338,15 @@ def load_gpxfile(xmlfile):
             is_with_alt = False
         retfile.hasAltitudes = is_with_alt
         return retfile
-    except Exception, err:
+    except Exception as err:
         logging.exception(err)
         return None
 
+
 def load_fitlogfile(xmlfile):
     """Load a fitlog file"""
     try:
-        if isinstance(xmlfile, basestring) and xmlfile.strip().startswith('<'):
+        if isinstance(xmlfile, str) and xmlfile.strip().startswith('<'):
             fid = xml.dom.minidom.parseString(xmlfile)
         else:
             fid = xml.dom.minidom.parse(xmlfile)
@@ -374,7 +366,7 @@ def load_fitlogfile(xmlfile):
         alt_count = 0
         name = u""
         for i in indices:
-            #get elevation
+            # get elevation
             (dist, lng, lat) = cps[i]
             sample = ptls[i]
             if sample.getAttribute('ele'):
@@ -396,18 +388,21 @@ def load_fitlogfile(xmlfile):
             is_with_alt = False
         retfile.hasAltitudes = is_with_alt
         return retfile
-    except Exception, err:
+    except Exception as err:
         logging.exception(err)
         return None
 
+
 def load_kmlfile(xmlfile):
     """Load a kml file"""
     try:
-        if isinstance(xmlfile, basestring) and xmlfile.strip().startswith('<'):
+        if isinstance(xmlfile, bytes) and xmlfile.strip().startswith(b'<'):
+            fid = xml.dom.minidom.parseString(xmlfile)
+        elif isinstance(xmlfile, str) and xmlfile.strip().startswith('<'):
             fid = xml.dom.minidom.parseString(xmlfile)
         else:
             fid = xml.dom.minidom.parse(xmlfile)
-            
+
         coordlist = []
         altlist = []
 
@@ -419,7 +414,7 @@ def load_kmlfile(xmlfile):
             data = data.replace('\n', ' ')
             coordlist += [el.split(',')[:2] for el in data.split(' ')]
 
-        #if LineString is not present try gx:coord (KML 2.2 ext)
+        # if LineString is not present try gx:coord (KML 2.2 ext)
         if len(coordlist) == 0:
             for gxcoord in fid.getElementsByTagName("gx:coord"):
                 node = gxcoord.childNodes[0]
@@ -438,8 +433,8 @@ def load_kmlfile(xmlfile):
         returncp = []
         for i in range(len(cps)):
             (dist, lng, lat) = cps[i]
-            #create self.num_cps check points
-            #print i
+            # create self.num_cps check points
+            # print i
             if i == 0 or dist > next_cp or i == len(cps)-1:
                 if is_with_alt:
                     # altitude is already in the kml file
@@ -455,10 +450,11 @@ def load_kmlfile(xmlfile):
         retfile = SlopeFile(returncp)
         retfile.hasAltitudes = is_with_alt
         return retfile
-    except Exception, err:
+    except Exception as err:
         logging.exception(err)
         return None
 
+
 def load_tcxfile(xmlfile):
     """Load a tcx file"""
     try:
@@ -469,7 +465,7 @@ def load_tcxfile(xmlfile):
         for sample in ptls:
             for ent in sample.childNodes:
                 if ent.localName == "Position":
-                    #Iterate on child nodes to get latitude and longitude
+                    # Iterate on child nodes to get latitude and longitude
                     for pos_node in ent.childNodes:
                         if pos_node.localName == "LatitudeDegrees":
                             lat = pos_node.childNodes[0].data
@@ -493,25 +489,27 @@ def load_tcxfile(xmlfile):
         logging.info("is with alt " + str(is_with_alt))
         retfile.hasAltitudes = is_with_alt
         return retfile
-    except Exception, err:
+    except Exception as err:
         logging.exception(err)
         return None
 
 (UNSUPPORTED, CPS, AUTHOR, DATA) = range(4)
+
+
 def __open_txt(file_, slopelist, slopenum):
     """Open a Cyclomaniac TXT file"""
-    if isinstance(file_, basestring):
-        fid = codecs.open(file_, encoding='utf-8', mode='r', errors='replace')
+    if isinstance(file_, str):
+        fid = open(file_, encoding='utf-8', mode='r', errors='replace')
     else:
         fid = file_
-    #with open(filename, 'rb') as fid:
+    # with open(filename, 'rb') as fid:
     mode = CPS
     for line in fid:
         line = line.strip()
-        #Find if switching to a different mode.
+        # Find if switching to a different mode.
         if line.startswith('['):
             modename = line.strip('[]').lower()
-            #print modename
+            # print modename
             if modename == 'author':
                 mode = AUTHOR
             elif modename == 'data':
@@ -520,21 +518,21 @@ def __open_txt(file_, slopelist, slopenum):
                 mode = UNSUPPORTED
             continue
 
-        #mode cps import check point to slope.
+        # mode cps import check point to slope.
         if mode == CPS:
-            #format is dist, alt, name, other.
+            # format is dist, alt, name, other.
             (dist, alt, name) = line.split(',')[:3]
             (dist, alt) = (float(dist), float(alt))
             name = name.strip('\"')
             slopelist.add_cp(slopenum, dist, alt, name)
 
-        #mode data imports extra information about the slope
+        # mode data imports extra information about the slope
         elif mode == DATA:
             try:
                 (lhs, rhs) = line.split('=')[:2]
             except:
                 continue
-            #print lhs +" <- " + rhs
+            # print lhs +" <- " + rhs
             if lhs == 'IDPercorso':
                 slopelist.set_name(slopenum, rhs)
             elif lhs == 'Stato':
@@ -543,17 +541,18 @@ def __open_txt(file_, slopelist, slopenum):
                 slopelist.set_url(slopenum, rhs)
     fid.close()
 
+
 def __open_sal(filepathname, slopelist, slopenum):
     """Open a Salitaker SAL file"""
-    if isinstance(filepathname, basestring):
+    if isinstance(filepathname, str):
         filename = os.path.split(filepathname)[-1]
         name = filename.split('.')[0]
         slopelist.set_name(slopenum, name)
 
-        fid = codecs.open(filepathname, encoding='utf-8', mode='r')
+        fid = open(filepathname, encoding='utf-8', mode='r')
     else:
         fid = filepathname
-    #with open(filepathname, 'rb') as fid:
+    # with open(filepathname, 'rb') as fid:
     ncp = fid.readline().strip().split(' ')[4]
     for _ in range(int(ncp)):
         name = fid.readline().strip()
@@ -568,56 +567,58 @@ def __open_crp(filepathname, slopelist, slopenum, num_cps=-1):
     Particular values are:
     0: import every cp
     -1: import the default number of cp"""
-    data = ""
-    if isinstance(filepathname, basestring):
-        fid = open(filepathname, mode='rb')#needed byte access not unicode string
+    data = b""
+    if isinstance(filepathname, str):
+        fid = open(filepathname, mode='rb')  # needed byte access not unicode string
     else:
         fid = filepathname
-    #with open(filepathname, 'rb') as fid:
-    byte = " "
+    # with open(filepathname, 'rb') as fid:
+    byte = b' '
     while byte:
         byte = fid.read(1)
         data += byte
     fid.close()
     if not data:
-        #File not accesible or empty
+        # File not accesible or empty
         return -1
 
-    if data[0] == '\x78' and data[1] == '\xda':
-        plaincsv = zlib.decompress(data)
-    elif data[0] == 'H' and data[1] == 'R':
-        plaincsv = data
+    if data[:2] == b'\x78\xda':
+        plaincsv = zlib.decompress(data).decode()
+    elif data[:2] == b'HR':
+        plaincsv = data.decode()
     else:
-        #Invalid file format
+        # Invalid file format
         return -1
-    #print plaincsv
+    # print(plaincsv)
     points = plaincsv.split('***')
-    line = points[0].split('\n')   #plaincsv.split('\n')
-    #print line[0]
+    line = points[0].split('\n')   # plaincsv.split('\n')
+    # print(line[0])
     next_cp = 0
     max_dist = float(line[-2].split('\t')[2])/100
     lastline = len(line)-3
     if num_cps < 0:
-        N = 20 #default value: often looks good.
+        N = 20  # default value: often looks good.
     elif num_cps == 0:
         N = lastline
     else:
         N = num_cps
     for i in range(lastline):
         fields = line[i+2].split('\t')
-        #print fields
+        # print fields
         dist = float(fields[2])/100
-        #create N check points
+        # create N check points
         if dist > next_cp or i == lastline or fields[8] != '':
             alt = float(fields[3])
-            slopelist.add_cp(slopenum, dist, alt, unicode(fields[8], 'utf-8', errors='replace'))
+            slopelist.add_cp(slopenum, dist, alt, fields[8])
             next_cp = dist + max_dist/N
     return True
 
-### Function that return kml string from gpx
+# Function that return kml string from gpx
+
+
 def gpxtokml(xmlfile):
     """Load a gpx file into kml"""
-    if isinstance(xmlfile, basestring) and xmlfile.strip().startswith('<'):
+    if isinstance(xmlfile, str) and xmlfile.strip().startswith('<'):
         fid = xml.dom.minidom.parseString(xmlfile)
     else:
         fid = xml.dom.minidom.parse(xmlfile)
@@ -634,7 +635,8 @@ def gpxtokml(xmlfile):
     kmlstring += "</coordinates></LineString>\n"
     kmlstring += "</Document>\n</kml>"
     return kmlstring
-### class for various formats
+# class for various formats
+
 
 class SlopeFile:
     def __init__(self, cps):
@@ -642,24 +644,29 @@ class SlopeFile:
         self.cps = cps
         self.cpToLoad = []
         self.hasAltitudes = False
+
     def min_dist(self):
         """ Minimum distance of the track """
         return self.cps[0][0]
+
     def max_dist(self):
         """ Maximum distance of the track """
         return self.cps[-1][0]
+
     def __len__(self):
         """ Number of points of the track """
         return len(self.cps)
+
     def newSlope(self, slopelist, slopenum, num, start_dist, end_dist, seltype=0):
         """ Create a new slope in the slopelist"""
         self.setcpToLoad(num, start_dist, end_dist, seltype)
         self.getCoords(slopelist, slopenum, start_dist, end_dist)
         for icp in self.cpToLoad:
-            (dist, alt, name, lat, lng)= self.cps[icp]
+            (dist, alt, name, lat, lng) = self.cps[icp]
             normadist = dist - float(start_dist)
             slopelist.add_cp(slopenum, normadist, alt, name)
         return True
+
     def setcpToLoad(self, num, start_dist, end_dist, seltype):
         """ Set cp index to load"""
         self.cpToLoad = []
@@ -677,10 +684,10 @@ class SlopeFile:
                 endindex = lencps - i
                 break
 
-        #print(startindex)
-        #print(endindex)
+        # print(startindex)
+        # print(endindex)
 
-        if seltype == 0:#using the given number of check points in num
+        if seltype == 0:  # using the given number of check points in num
             max_dist = self.max_dist()
             next_cp = 0.0
 
@@ -693,11 +700,11 @@ class SlopeFile:
 
             for i in range(startindex, endindex):
                 dist = self.cps[i][0]
-                #create approximately N check points
+                # create approximately N check points
                 if i == startindex or dist > next_cp or i == endindex-1:
                     self.cpToLoad.append(i)
                     next_cp = dist + max_dist/N
-        elif seltype == 1:#using the passed minimum distance in metres in num
+        elif seltype == 1:  # using the passed minimum distance in metres in num
             next_dist = num
             for i in range(startindex, endindex):
                 dist = self.cps[i][0]
@@ -705,7 +712,7 @@ class SlopeFile:
                     self.cpToLoad.append(i)
                     next_dist = dist + num
         return True
-        
+
     def getCoords(self, slopelist, slopenum, start_dist, end_dist):
         """ Get coords from cps to the passed slope"""
         startindex = 0
@@ -721,10 +728,10 @@ class SlopeFile:
                 endindex = lencps - i
                 break
 
-        #print(startindex)
-        #print(endindex)
+        # print(startindex)
+        # print(endindex)
         for i in range(startindex, endindex):
-            (dist, alt, name, lat, lng)= self.cps[i]
+            (dist, alt, name, lat, lng) = self.cps[i]
             slopelist.add_coord(slopenum, float(lat), float(lng))
 
 # vim:sw=4:softtabstop=4:expandtab
