@@ -1,8 +1,8 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 """launcher.py """
 
-# Copyright (C) 2008, 2009, 2010, 2011 Federico Brega, Pierluigi Villani
+# Copyright (C) 2008-2014 Federico Brega, Pierluigi Villani
 
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -15,26 +15,28 @@
 # GNU General Public License for more details.
 
 import sys
-import locale, gettext
+import locale
+import gettext
 import os
 import glal_selected
 
+
 def setup_gettext():
     """Application translation system setup"""
 
-    #Setup translation.
+    # Setup translation.
     langs = []
     mylocale = locale.getdefaultlocale()[0]
     if mylocale:
         langs = [mylocale]
-    #Other languages in the system.
+    # Other languages in the system.
     language = os.environ.get('LANGUAGE', None)
     if language:
         langs += language.split(':')
-    #If there isn't any better use this.
+    # If there isn't any better use this.
     langs += ["en_CA", "en_US"]
     po_path = os.path.join(os.path.dirname(os.path.abspath(__file__)),
-                                           '..','po')
+                                           '..', 'po')
 
     if not os.path.isdir(po_path):
         po_path = "/usr/share/locale"
@@ -42,15 +44,16 @@ def setup_gettext():
     gettext.textdomain("cyclograph")
     lang = gettext.translation("cyclograph", po_path,
                                      languages=langs, fallback=True)
-    lang.install(unicode=True)
+    lang.install()
+
 
 def main():
     setup_gettext()
     if len(sys.argv) < 2:
-        #If there has been an error with the commandline specified frontend
-        #try others in the order (PyQt gets first because is the supported
-        #frontend on Mac Os and Windows where users might want a graphical
-        #interface.
+        # If there has been an error with the commandline specified frontend
+        # try others in the order (PyQt gets first because is the supported
+        # frontend on Mac Os and Windows where users might want a graphical
+        # interface.
         if glal_selected.use('qt'):
             import cyclograph
             cyclograph.main_qt()
@@ -63,7 +66,7 @@ def main():
         else:
             sys.exit(2)
     else:
-        #also file=* is an argument even if it is not space separed.
+        # also file=* is an argument even if it is not space separed.
         arguments = set([token.partition('=')[0] for token in sys.argv])
         if arguments & set(('-q', '--qt')):
             if glal_selected.use('qt') != 'qt':
