@@ -1,8 +1,8 @@
 # -*- coding: utf-8 -*-
-#themes.py
+# themes.py
 """This module provides a graphic theme for plotting"""
 
-# Copyright (C) 2010, 2011, 2012, 2013 Federico Brega, Pierluigi Villani
+# Copyright (C) 2010-2015 Federico Brega, Pierluigi Villani
 
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -14,19 +14,20 @@
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 
-from __future__ import unicode_literals
 
 class ThemeManager:
     def getthemeslist(self):
         """Return names associated with classes subclassing Theme."""
         return sorted(Theme.themes.keys())
+
     def gettheme(self, name):
         """Return reference to the given class name."""
         if name in Theme.themes:
             return Theme.themes[name]
         return None
 
-class ThemeMount(type): #from http://martyalchin.com/2008/jan/10/simple-plugin-framework/
+
+class ThemeMount(type):  # from http://martyalchin.com/2008/jan/10/simple-plugin-framework/
     def __init__(cls, name, bases, attrs):
         if not hasattr(cls, 'themes'):
             # This branch only executes when processing the mount point itself.
@@ -38,78 +39,98 @@ class ThemeMount(type): #from http://martyalchin.com/2008/jan/10/simple-plugin-f
             # This must be a plugin implementation, which should be registered.
             # Simply appending it to the list is all that's needed to keep
             # track of it later.
-            #if cls.themename in cls.themes:
+            # if cls.themename in cls.themes:
                 # TODO: if user can create/install themes warn if a name is
                 # used by more than one theme
-                #return
+                # return
             cls.themes[cls.themename] = cls
 
 
-class Theme:
+class Theme(metaclass=ThemeMount):
     """Model of a theme: every theme must subclass this class."""
-    __metaclass__ = ThemeMount
     themename = ""
     shear = -0.15
+
     @staticmethod
     def getcolor(colors, levels, grad):
         """Return gradient relative color"""
         pass
+
     @staticmethod
     def drawslopeinfo(devc, info, settings, startx, starty):
         """Draw slope's name and other info background"""
         pass
+
+    @staticmethod
+    def drawlegend(devc, settings, startx, starty, title):
+        """Draw slope's legend"""
+        pass
+
     @staticmethod
     def paintbackground(devc, width, height):
         """Draw background"""
         pass
+
     @staticmethod
     def gradback(devc, width, height, setting_f, startcol="rgb(64, 185, 255)", endcol='white'):
         """Draw a gradient background"""
         pass
+
     @staticmethod
     def olines(devc, slope, margins, width, height, metersize):
         """Draw altitude line"""
         pass
+
     @staticmethod
     def alttext(devc, slope, margins, max_y, metersize):
         """draw altitude text aside"""
         pass
+
     @staticmethod
     def yaxis(devc, slope, margins, max_y, metersize):
         """Draw y axis"""
         pass
+
     @staticmethod
     def xaxis(devc, slope, margins, max_y, d_incr, kmsize, dx, dy):
         """Draw km bar"""
         pass
+
     @staticmethod
     def fillhslopecontour(devc, spath_back, dx, dy, colorlist):
         """ Draw horizontal contour."""
         pass
+
     @staticmethod
     def fillhpoly(devc, points, color):
         """fill horizontal polygons of the 3d slope"""
         pass
+
     @staticmethod
     def fillfirsthpoly(devc, points, color):
         """fill first horizontal polygon of the 3d slope"""
+
     @staticmethod
     def fillvslopecontour(devc, path_points, pixel_0m, pixel_1000m, vpolygons, colorlist):
         """Draw a close smooth path thant contais the whole slope."""
         pass
+
     @staticmethod
     def fillvpoly(devc, points, color):
         """fill vertical polygons (2d and 3d) of the slope"""
         pass
+
     @staticmethod
     def desctext(devc, text, pos_x, pos_y, font):
         """Draw description text for every cp"""
         pass
+
     @staticmethod
     def gradtext(devc, text, pos_x, pos_y, font):
         """Draw gradient text for every cp"""
         pass
 
+
 class ThemeClassic(Theme):
     """Classic CycloGraph theme"""
     themename = "Classic"
@@ -137,6 +158,24 @@ class ThemeClassic(Theme):
             devc.drawtext(eleg, startx+10, starty+60)
             devc.drawtext(heightg, startx+10, starty+80)
 
+    @staticmethod
+    def drawlegend(devc, settings, startx, starty, title):
+        """Draw slope's legend"""
+        devc.setbrush("rgb(236, 236, 236)")
+        rec = (startx, starty, 310, 52)
+        devc.drawrectangle(*rec)
+        devc.setfont(settings['fdesc'])
+        devc.setpen('black', 2)
+        devc.drawtext(title, startx+130, starty+2)
+        incr = 43
+        devc.setpen('black', 1)
+        for i in range(len(settings['colors'])):
+            devc.setbrush(settings['colors'][i])
+            rec = (startx+8+i*incr, starty+20, 36, 10)
+            devc.drawrectangle(*rec)
+            alti_t = (">%d%%" % (settings['levels'][i]))
+            devc.drawtext(alti_t, startx+9+i*incr, starty+34)
+
     @staticmethod
     def paintbackground(devc, width, height):
         """Draw backgroundr"""
@@ -192,7 +231,7 @@ class ThemeClassic(Theme):
         linesnum = int(slope.max_h / slope.h_incr)
         devc.drawline(lef_mar,  max_y - low_mar + 4,
                       lef_mar,  max_y - low_mar - 8 \
-                      - int(linesnum * slope.h_incr - slope.min_h )* metersize)
+                      - int(linesnum * slope.h_incr - slope.min_h) * metersize)
 
     @staticmethod
     def xaxis(devc, slope, margins, max_y, d_incr, kmsize, dx, dy):
@@ -200,15 +239,15 @@ class ThemeClassic(Theme):
         (updownmar, leftrightmar) = margins
         (lef_mar, rig_mar) = leftrightmar
         (upp_mar, low_mar) = updownmar
-        for i in range(int(slope.max_d) / d_incr - int(slope.min_d) / d_incr + 1):
-            if i % 2 == 0 :
+        for i in range(int(int(slope.max_d) / d_incr - int(slope.min_d) / d_incr + 1)):
+            if i % 2 == 0:
                 devc.setbrush("rgb(0, 0, 0)")
-            else :
+            else:
                 devc.setbrush("rgb(255, 255, 255)")
             rec = (i * d_incr * kmsize + lef_mar + dx,
                    max_y - low_mar + dy, d_incr * kmsize + 1, 5)
             devc.drawrectangle(*rec)
-            d_label = ("%d" % (i * d_incr +int(slope.min_d)))
+            d_label = ("%d" % (i * d_incr + int(slope.min_d)))
             devc.drawtext(d_label, rec[0], rec[1] + 5)
 
     @staticmethod
@@ -240,11 +279,13 @@ class ThemeClassic(Theme):
         """Draw gradient text for every cp"""
         devc.setfont(font)
         devc.drawtext(text, pos_x, pos_y)
-        
+
+
 class ThemeBclassic(Theme):
     """Bezier classic CycloGraph theme"""
     themename = "Smooth classic"
     shear = -0.15
+
     @staticmethod
     def getcolor(colors, levels, grad):
         """Return gradient relative color"""
@@ -253,11 +294,17 @@ class ThemeBclassic(Theme):
             if gradient < levels[i]:
                 return colors[i-1]
         return colors[i]
+
     @staticmethod
     def drawslopeinfo(devc, info, settings, startx, starty):
         """Draw slope's name and other info background"""
         ThemeClassic.drawslopeinfo(devc, info, settings, startx, starty)
 
+    @staticmethod
+    def drawlegend(devc, settings, startx, starty, title):
+        """Draw slope's legend"""
+        ThemeClassic.drawlegend(devc, settings, startx, starty, title)
+
     @staticmethod
     def paintbackground(devc, width, height):
         """Draw backgroundr"""
@@ -319,7 +366,7 @@ class ThemeBclassic(Theme):
         index = 0
         start_pnt = path_points.pop(0)
         point = path_points[0]
-        start_pnt = point[0] 
+        start_pnt = point[0]
         for pnt in path_points[:-1]:
             devc.setbrush(colorlist[index])
             path = devc.startpath(start_pnt)
@@ -341,6 +388,7 @@ class ThemeBclassic(Theme):
         """Draw gradient text for every cp"""
         ThemeClassic.gradtext(devc, text, pos_x, pos_y, font)
 
+
 class ThemeBwhite(Theme):
     """bezier white theme"""
     themename = "giro"
@@ -364,7 +412,7 @@ class ThemeBwhite(Theme):
     @staticmethod
     def gradback(devc, width, height, setting_f, startcol="rgb(0, 0, 0)", endcol="rgb(70, 80, 70)"):
         """Draw a gradient background"""
-        devc.setfont(setting_f)#FIXME: move setfont in a new method
+        devc.setfont(setting_f)  # FIXME: move setfont in a new method
 
     @staticmethod
     def alttext(devc, slope, margins, max_y, metersize):
@@ -429,6 +477,7 @@ class ThemeBwhite(Theme):
         """Draw description text for every cp"""
         ThemeClassic.desctext(devc, text, pos_x, pos_y, font)
 
+
 class ThemeBlue(Theme):
     """Blue theme"""
     themename = "Blue"
@@ -525,7 +574,7 @@ class ThemeBlue(Theme):
     def fillfirsthpoly(devc, points, color):
         """fill first horizontal polygon of the 3d slope"""
         devc.setpen('black', 1)
-        #devc.setbrush(color)
+        # devc.setbrush(color)
         devc.setlineargradientbrush([("rgb(250, 250, 250)", 0),
                                      ("rgb(200, 200, 250)", 1)],
                                     (0, points[3][1]), (0, devc.size_y/2))
@@ -535,10 +584,10 @@ class ThemeBlue(Theme):
     def fillvpoly(devc, points, color):
         """fill vertical polygons (2d and 3d) of the slope"""
         devc.setpen('black', 1)
-        #devc.setbrush(color)
+        # devc.setbrush(color)
         devc.setlineargradientbrush([("rgb(250, 250, 250)", 0),
                                      ("rgb(200, 200, 250)", 1)],
-                                    (0, points[1][1]), (0, devc.size_y/2))#max(points[3][1], points[2][1])))
+                                    (0, points[1][1]), (0, devc.size_y/2))  # max(points[3][1], points[2][1])))
         devc.drawpolygon(points)
 
     @staticmethod
@@ -555,6 +604,7 @@ class ThemeBlue(Theme):
         devc.setfont(font)
         devc.drawtext(text, pos_x, pos_y)
 
+
 class ThemeBlack(Theme):
     """Black theme"""
     themename = "Black"
@@ -583,6 +633,26 @@ class ThemeBlack(Theme):
             devc.drawtext(eleg, startx+10, starty+60)
             devc.drawtext(heightg, startx+10, starty+80)
 
+    @staticmethod
+    def drawlegend(devc, settings, startx, starty, title):
+        """Draw slope's legend"""
+        devc.setbrush("rgb(90, 100, 90)")
+        rec = (startx, starty, 310, 52)
+        devc.setpen('black', 1)
+        devc.drawrectangle(*rec)
+        devc.setfont(settings['fdesc'])
+        devc.setpen('white', 2)
+        devc.drawtext(title, startx+130, starty+2)
+        incr = 43
+        for i in range(len(settings['colors'])):
+            devc.setbrush(settings['colors'][i])
+            devc.setpen('black', 1)
+            rec = (startx+8+i*incr, starty+20, 36, 10)
+            devc.drawrectangle(*rec)
+            devc.setpen('white', 1)
+            alti_t = (">%d%%" % (settings['levels'][i]))
+            devc.drawtext(alti_t, startx+9+i*incr, starty+34)
+
     @staticmethod
     def paintbackground(devc, width, height):
         """Draw background"""
@@ -624,17 +694,17 @@ class ThemeBlack(Theme):
         (updownmar, leftrightmar) = margins
         (lef_mar, rig_mar) = leftrightmar
         (upp_mar, low_mar) = updownmar
-        for i in range(int(slope.max_d) / d_incr - int(slope.min_d) / d_incr + 1):
-            if i % 2 == 0 :
+        for i in range(int(int(slope.max_d) / d_incr - int(slope.min_d) / d_incr + 1)):
+            if i % 2 == 0:
                 devc.setbrush("rgb(0, 0, 0)")
-            else :
+            else:
                 devc.setbrush("rgb(255, 255, 255)")
             rec = (i * d_incr * kmsize + lef_mar + dx,
                    max_y - low_mar + dy, d_incr * kmsize + 1, 5)
             devc.setpen('black', 1)
             devc.drawrectangle(*rec)
             devc.setpen('white', 1)
-            d_label = ("%d" % (i * d_incr +int(slope.min_d)))
+            d_label = ("%d" % (i * d_incr + int(slope.min_d)))
             devc.drawtext(d_label, rec[0], rec[1] + 5)
 
     @staticmethod
@@ -678,6 +748,7 @@ class ThemeBlack(Theme):
         devc.setfont(font)
         devc.drawtext(text, pos_x, pos_y)
 
+
 class ThemeGray(Theme):
     """Gray theme"""
     themename = "Gray"
@@ -686,10 +757,10 @@ class ThemeGray(Theme):
     @staticmethod
     def getcolor(colors, levels, grad):
         """Return gradient relative color"""
+        gradient = abs(grad)
         graycolors = ["rgb(175,175,175)", "rgb(150,150,150)",
                       "rgb(125,125,125)", "rgb(100,100,100)",
                       "rgb(75,75,75)", "rgb(50,50,50)", "rgb(25,25,25)"]
-        gradient = abs(grad)
         for i in range(len(levels)):
             if gradient < levels[i]:
                 return graycolors[i-1]
@@ -700,6 +771,27 @@ class ThemeGray(Theme):
         """Draw slope's name and other info background"""
         ThemeClassic.drawslopeinfo(devc, info, settings, startx, starty)
 
+    @staticmethod
+    def drawlegend(devc, settings, startx, starty, title):
+        """Draw slope's legend"""
+        graycolors = ["rgb(175,175,175)", "rgb(150,150,150)",
+                      "rgb(125,125,125)", "rgb(100,100,100)",
+                      "rgb(75,75,75)", "rgb(50,50,50)", "rgb(25,25,25)"]
+        devc.setbrush("rgb(236, 236, 236)")
+        rec = (startx, starty, 310, 52)
+        devc.drawrectangle(*rec)
+        devc.setfont(settings['fdesc'])
+        devc.setpen('black', 2)
+        devc.drawtext(title, startx+130, starty+2)
+        incr = 43
+        devc.setpen('black', 1)
+        for i in range(len(graycolors)):
+            devc.setbrush(graycolors[i])
+            rec = (startx+8+i*incr, starty+20, 36, 10)
+            devc.drawrectangle(*rec)
+            alti_t = (">%d%%" % (settings['levels'][i]))
+            devc.drawtext(alti_t, startx+9+i*incr, starty+34)
+
     @staticmethod
     def paintbackground(devc, width, height):
         """Draw background"""
@@ -710,7 +802,6 @@ class ThemeGray(Theme):
     @staticmethod
     def gradback(devc, width, height, setting_f, startcol="rgb(0, 0, 0)", endcol="rgb(70, 80, 70)"):
         """Draw a gradient background"""
-        area = (0, 0, width, height)
         devc.setpen('black', 1)
         devc.setfont(setting_f)
 
@@ -790,4 +881,5 @@ class ThemeGray(Theme):
         devc.drawtext(text, pos_x + 1, pos_y + 1)
         devc.setpen('white', 1)
         devc.drawtext(text, pos_x, pos_y)
+
 # vim:sw=4:softtabstop=4:expandtab
